﻿using System;
using System.Collections.Generic;
using TSLab.Script.Helpers;

namespace TSLab.Script.Handlers
{
    [HandlerCategory("TradeMath")]
    public class Max : IDoubleAccumHandler, IDouble2CalculatorHandler
    {
        #region IDouble2CalculatorHandler Members

        public double Execute(double source1, double source2)
        {
            return Math.Max(source1, source2);
        }

        #endregion

        #region IDoubleAccumHandler Members

        public IList<double> Execute(IList<double> source1, IList<double> source2)
        {
            return Series.Max(source1, source2);
        }

        #endregion
    }

    [HandlerCategory("TradeMath")]
    public class Min : IDoubleAccumHandler, IDouble2CalculatorHandler
    {
        #region IDouble2CalculatorHandler Members

        public double Execute(double source1, double source2)
        {
            return Math.Min(source1, source2);
        }

        #endregion

        #region IDoubleAccumHandler Members

        public IList<double> Execute(IList<double> source1, IList<double> source2)
        {
            return Series.Min(source1, source2);
        }

        #endregion
    }

    [HandlerCategory("TradeMath")]
    public class Add : IDoubleAccumHandler, IDouble2CalculatorHandler
    {
        #region IDouble2CalculatorHandler Members

        public double Execute(double source1, double source2)
        {
            return source1 + source2;
        }

        #endregion

        #region IDoubleAccumHandler Members

        public IList<double> Execute(IList<double> source1, IList<double> source2)
        {
            return Series.Add(source1, source2);
        }

        #endregion
    }

    [HandlerCategory("TradeMath")]
    public class Sub : IDoubleAccumHandler, IDouble2CalculatorHandler
    {
        #region IDouble2CalculatorHandler Members

        public double Execute(double source1, double source2)
        {
            return source1 - source2;
        }

        #endregion

        #region IDoubleAccumHandler Members

        public IList<double> Execute(IList<double> source1, IList<double> source2)
        {
            return Series.Sub(source1, source2);
        }

        #endregion
    }

    [HandlerCategory("TradeMath")]
    public class Not : IBoolConvertor
    {
        #region IBoolConvertor Members

        public IList<bool> Execute(IList<bool> src)
        {
            var dst = new bool[src.Count];
            for (int i = 0; i < src.Count; i++)
            {
                dst[i] = !src[i];
            }
            return dst;
        }

        public bool Execute(bool source, int num)
        {
            return !source;
        }

        #endregion
    }

    [HandlerCategory("TradeMath")]
    public class And : IBoolAccum
    {
        #region IBoolAccum Members

        public IList<bool> Execute(IList<bool> src1, IList<bool> src2)
        {
            int count = Math.Max(src1.Count, src2.Count);
            int mcount = Math.Min(src1.Count, src2.Count);
            var dst = new bool[count];
            for (int i = 1; i < mcount; i++)
            {
                dst[i] = (src1[i] && src2[i]);
            }
            return dst;
        }

        public bool Execute(bool source1, bool source2)
        {
            return source1 && source2;
        }

        #endregion
    }

    [HandlerCategory("TradeMath")]
    public class Or : IBoolAccum
    {
        #region IBoolAccum Members

        public IList<bool> Execute(IList<bool> src1, IList<bool> src2)
        {
            int count = Math.Max(src1.Count, src2.Count);
            var dst = new bool[count];
            for (int i = 1; i < count; i++)
            {
                bool s1 = src1.Count != 0 && src1[Math.Min(i, src1.Count - 1)];
                bool s2 = src2.Count != 0 && src2[Math.Min(i, src2.Count - 1)];
                dst[i] = s1 || s2;
            }
            return dst;
        }

        public bool Execute(bool source1, bool source2)
        {
            return source1 || source2;
        }

        #endregion
    }

    [HandlerName("Cross Over")]
    [HandlerCategory("TradeMath")]
    public class CrossOver : IDoubleCompaperHandler
    {
        #region IDoubleCompaperHandler Members

        public IList<bool> Execute(IList<double> src1, IList<double> src2)
        {
            int count = Math.Max(src1.Count, src2.Count);
            int mcount = Math.Min(src1.Count, src2.Count);
            var dst = new bool[count];
            for (int i = 1; i < mcount; i++)
            {
                dst[i] = (src1[i] > src2[i] && src1[i - 1] <= src2[i - 1]);
            }
            return dst;
        }

        #endregion
    }

    [HandlerName("Cross Under")]
    [HandlerCategory("TradeMath")]
    public class CrossUnder : IDoubleCompaperHandler
    {
        #region IDoubleCompaperHandler Members

        public IList<bool> Execute(IList<double> src1, IList<double> src2)
        {
            int count = Math.Max(src1.Count, src2.Count);
            int mcount = Math.Min(src1.Count, src2.Count);
            var dst = new bool[count];
            for (int i = 1; i < mcount; i++)
            {
                dst[i] = (src1[i] < src2[i] && src1[i - 1] >= src2[i - 1]);
            }
            return dst;
        }

        #endregion
    }
}
